#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <util/delay.h>

// control line pins
#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

// macros for control lines
#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}
/*************************************************/
void write_command(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTE=command>>8;
	PORTA=command;

	CLR_WR;
	SET_WR;
	SET_CS;
}

/*************************************************/
void write_data(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTE=data>>8;
	PORTA=data;

	CLR_WR;
	SET_WR;
	SET_CS;
}
/*************************************************/
void initialization()
{
	SET_RD;
	SET_WR;
	SET_CS;
	SET_CD;
	PORTA=0x00;
	PORTE=0x00;

	CLR_RESET;
	delay(200);
	SET_RESET;
	delay(500);

	write_command(0x0011); // Exit Sleep
	delay(20);

	write_command(0x0026); //Set Default Gamma
	write_data(0x0004);

	write_command(0x00b1);//Set Frame Rate
	write_data(0x000b);
	write_data(0x0014);

	write_command(0x00c0);//Set VRH1[4:0] & VC[2:0] for VCI1 & GVDD
	write_data(0x0008);
	write_data(0x0000);

	write_command(0x00c1); //Set BT[2:0] for AVDD & VCL & VGH & VGL
	write_data(0x0005);

	write_command(0x00c5); //Set VMH[6:0] & VML[6:0] for VOMH & VCOML
	write_data(0x0041);
	write_data(0x0030);

	write_command(0x00c7);
	write_data(0x00c1);

	write_command(0x0036); //Set Scanning Direction
	write_data(0x00c8);

	write_command(0x00ec);//Set pumping clock frequency 
	write_data(0x001b);

	write_command(0x003a); //Set Color Format  
	write_data(0x0055);

	write_command(0x00f2); //E0h & E1h Enable/Disable
	write_data(0x0001);

	write_command(0x00e0); // P GAM
	write_data(0x0028); //P1
	write_data(0x0024); //P2
	write_data(0x0022); //P3
	write_data(0x0031); //P4
	write_data(0x002b); //P5
	write_data(0x000e); //P6
	write_data(0x0053); //P7
	write_data(0x00a5); //P8
	write_data(0x0042); //P9
	write_data(0x0016); //P10
	write_data(0x0018); //P11
	write_data(0x0012); //P12
	write_data(0x001a); //P13
	write_data(0x0014); //P14
	write_data(0x0003); //P15

	write_command(0x00e1); // N GAM
	write_data(0x0017); //P1
	write_data(0x001b); //P2
	write_data(0x001d); //P3
	write_data(0x000e); //P4
	write_data(0x0014); //P5
	write_data(0x0011); //P6
	write_data(0x002c); //P7
	write_data(0x00a5); //P8
	write_data(0x003d); //P9
	write_data(0x0009); //P10
	write_data(0x0027); //P11
	write_data(0x002d); //P12
	write_data(0x0025);  //P13
	write_data(0x002b);  //P14
	write_data(0x003c); //P15 

	write_command(0x002A); //еַѡ
	write_data(0x0000);
	write_data(0x0000);
	write_data(0x0000);
	write_data(0x007f); //SET 127

	write_command(0x002B);  // ҳѡ
	write_data(0x0000);
	write_data(0x0000);
	write_data(0x0000);
	write_data(0x009f);//SET159

	write_command(0x0029); // Display On
}
/*************************************************/
void Display_Home()
{
	write_command(0x002a); // GRAM Address Set (Horizontal Address) (R20h)
	write_data(0x0000);
	write_command(0x002b); // GRAM Address Set (Vertical Address) (R21h)
	write_data(0x0000);
	write_command(0x002C);  // Write Data to GRAM (R2Ch)
}
/*************************************************/
void display_rgb(unsigned int data)
{
	unsigned int i,j;
	Display_Home();
	write_command(0x22);
	for(i=0;i<160;i++)
	{  
		for(j=0;j<128;j++)
		{
			write_data(data);
		}
	}
}
/*************************************************/
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"
	display_rgb(BLACK);

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 128 * 160; // total # of pixels to write

	Display_Home();
	write_command(0x2C);

	j = 0; // byte count
	
	while (1)
	{
		write_data(buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<30;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/
void LCD_test()
{
	unsigned int i,j;
	Display_Home();

	for(i=0;i<160;i++)
	for(j=0;j<128;j++)
	{
		if(i>139)write_data(BLACK);
		else if(i>119)write_data(MAGENTA);
		else if(i>99)write_data(BLUE);
		else if(i>79)write_data(CYAN);
		else if(i>59)write_data(GREEN);
		else if(i>39)write_data(YELLOW);
		else if(i>19)write_data(RED);
		else write_data(WHITE);

	}
}
/*************************************************/
int main()
{
	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRA = 0xFF;    	// low data port, all outputs
	DDRB = 0xFF;    	// low data port, all outputs
	DDRE = 0xFF;    	// low data port, all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
	initialization();

	LCD_test();

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card

	delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1)
	{
	}
	return 0;

}
/*************************************************/


